### Code by Sophia von Stockert, 2017


library(foreign)
library(NetworkComparisonTest)
library(qgraph)
library(bootnet)
library(mgm)

#load bootstrapped data and NCT results
load(file="boot_Spear_T1_all.Rdata")
load(file="boot_Spear_T1_all_C")
load(file="boot_Spear_T1_611.Rdata")
load(file="boot_Spear_T1_611_C")
load(file="boot_Spear_T2_611.Rdata")
load(file="boot_Spear_T2_611_C")
load(file="NCT_T1_611_T2_611_Pearson.Rdata")

#load and prepare data
D_PTSD<-read.spss("PTSD2017code.sav", to.data.frame = T)
View(D_PTSD)
summary(D_PTSD)
str(D_PTSD)

#make wave 2 items numeric
D_PTSD[, 31:50]<-lapply(D_PTSD[, 31:50], as.numeric)#results in numbers from 1 to 5
summary(D_PTSD)
str(D_PTSD)

#check data distribution
hist(D_PTSD[, 11:30])#wave 1 data is skewed
hist(D_PTSD[1:611, 31:50])#wave 2 data is skewed

SymptomNames<-c("Intrusive Thoughts", "Nightmares","Flashbacks","Emotional Cue Reactivity","Physical Cue Reactivity","Avoidance of Thoughts","Avoidance of Reminders","Trauma-related Amnesia","Negative Beliefs","Blame of self or others","Negative Trauma-related Emotions","Loss of Interest","Detachment","Restricted Affect","Irritability/Anger","Self-destructive/reckless behavior","Hypervigilance","Exaggerated Startle Response", "Difficulty Concentrating", "Sleep Disturbance")
LabelsPTSD<-c("B1", "B2", "B3", "B4", "B5", "C1", "C2", "D1", "D2", "D3", "D4", "D5", "D6", "D7", "E1", "E2", "E3", "E4", "E5", "E6")
GroupLabels <- list("Intrusions" = c(1:5), "Avoidance" = c(6:7), "Cognition & Mood Alteration" = c(8:14), "Arousal & Reactivity Alterations" = c(15:20))  
Dat1<-plyr::rename(D_PTSD, c("W1_PCL1"="1", "W1_PCL2"="2", "W1_PCL3"="3", "W1_PCL4"="4", "W1_PCL5"="5", "W1_PCL6"="6", "W1_PCL7"="7", "W1_PCL8"="8", "W1_PCL9"="9", "W1_PCL10"="10", "W1_PCL11"="11", "W1_PCL12"="12", "W1_PCL13"="13", "W1_PCL14"="14", "W1_PCL15"="15", "W1_PCL16"="16", "W1_PCL17"="17", "W1_PCL18"="18", "W1_PCL19"="19", "W1_PCL20"="20"))
Dat2<-plyr::rename(D_PTSD, c("W2_PCL1"="1", "W2_PCL2"="2", "W2_PCL3"="3", "W2_PCL4"="4", "W2_PCL5"="5", "W2_PCL6"="6", "W2_PCL7"="7", "W2_PCL8"="8", "W2_PCL9"="9", "W2_PCL10"="10", "W2_PCL11"="11", "W2_PCL12"="12", "W2_PCL13"="13", "W2_PCL14"="14", "W2_PCL15"="15", "W2_PCL16"="16", "W2_PCL17"="17", "W2_PCL18"="18", "W2_PCL19"="19", "W2_PCL20"="20"))




#most participants in this data set score "not at all" and "a little bit" - Spearman correlations can handle this better than polychoric correlations
#estimate Spearman network, weights matrix and centrality at T1 with n=1268
SNetwork_T1_all<-qgraph(cor(Dat1[ , 11:30], method="spearman"), layout="spring", graph="glasso", sampleSize=nrow(D_PTSD), cut=0, details=T, vsize=6.5, nodeNames=SymptomNames, groups = GroupLabels, palette = "colorblind", labels = LabelsPTSD, legend.cex=.4, posCol="darkblue", negCol="red", title="Spearman_T1_all")#strongest edge=0.38
Cent1<-centrality(SNetwork_T1_all)
Cent1$OutDegree
which.max(Cent1$OutDegree)
WM_T1_all <- getWmat(SNetwork_T1_all)

#Spearman bootstrap (edge weight accuracy)
Network_Boot_T1_all<-estimateNetwork(Dat1[ , 11:30], default="EBICglasso", corMethod = "cor", corArgs=list(method = "spearman", use = "pairwise.complete.obs"))
boot_Spear_T1_all<-bootnet(Network_Boot_T1_all, nBoots=1000, nCores=8, verbose=T)
plot(boot_Spear_T1_all, order="sample", labels=F)
save(boot_Spear_T1_all, file="boot_Spear_T1_all.Rdata")
#Spearman bootstrap (centrality stability)
boot_Spear_T1_all_C<-bootnet(Network_Boot_T1_all, nBoots=1000, nCores=8, type="case", verbose=T)
plot(boot_Spear_T1_all_C)
corStability(boot_Spear_T1_all_C)
save(boot_Spear_T1_all_C, file="boot_Spear_T1_all_C")
#for difference
plot(boot_Spear_T1_all, "edge", plot="difference", onlyNonZero = T, order="sample")
plot(boot_Spear_T1_all, "strength")


#estimate Spearman network, weights matrix and centrality at T1 with n=611
SNetwork_T1_611<-qgraph(cor(Dat1[1:611, 11:30], method="spearman"), layout="spring", graph="glasso", sampleSize=nrow(D_PTSD), cut=0, details=T, vsize=6.5, nodeNames=SymptomNames, groups = GroupLabels, palette = "colorblind", labels = LabelsPTSD, legend.cex=.4, posCol="darkblue", negCol="red", title="Spearman_T1_611")#strongest edge=0.32
Cent1b<-centrality(SNetwork_T1_611)
Degree_T1_611<-Cent1b$OutDegree
which.max(Cent1b$OutDegree)
WM_T1_611 <- getWmat(SNetwork_T1_611)

#Spearman bootstrap (edge weight accuracy)
Network_Boot_T1_611<-estimateNetwork(Dat1[1:611, 11:30], default="EBICglasso", corMethod = "cor", corArgs=list(method = "spearman", use = "pairwise.complete.obs"))
boot_Spear_T1_611<-bootnet(Network_Boot_T1_611, nBoots=1000, nCores=8, verbose=T)
plot(boot_Spear_T1_611, order="sample", labels=F)
save(boot_Spear_T1_611, file="boot_Spear_T1_611.Rdata")
#Spearman bootstrap (centrality stability)
boot_Spear_T1_611_C<-bootnet(Network_Boot_T1_611, nBoots=1000, nCores=8, type="case", verbose=T)
plot(boot_Spear_T1_611_C)
corStability(boot_Spear_T1_611_C)
save(boot_Spear_T1_611_C, file="boot_Spear_T1_611_C")
#for difference
plot(boot_Spear_T1_611, "edge", plot="difference", onlyNonZero = T, order="sample")
plot(boot_Spear_T1_611, "strength")


#estimate Spearman network, weights matrix and centrality at T2 with n=611
SNetwork_T2_611<-qgraph(cor(Dat2[1:611, 31:50], method="spearman"), layout="spring", graph="glasso", sampleSize=nrow(D_PTSD), cut=0, details=T, vsize=6.5, nodeNames=SymptomNames, groups = GroupLabels, palette = "colorblind", labels = LabelsPTSD, legend.cex=.4, posCol="darkblue", negCol="red", title="Spearman_T2_611")#biggest edge=0.46
Cent2<-centrality(SNetwork_T2_611)
Degree_T2_611<-Cent2$OutDegree
which.max(Cent2$OutDegree)
WM_T2_611 <- getWmat(SNetwork_T2_611)

#Spearman bootstrap (edge weight accuracy)
Network_Boot_T2_611<-estimateNetwork(Dat2[1:611, 31:50], default="EBICglasso", corMethod = "cor", corArgs=list(method = "spearman", use = "pairwise.complete.obs"))
boot_Spear_T2_611<-bootnet(Network_Boot_T2_611, nBoots=1000, nCores=8, verbose=T)
plot(boot_Spear_T2_611, order="sample", labels=F)
save(boot_Spear_T2_611, file="boot_Spear_T2_611.Rdata")
#Spearman bootstrap (centrality stability)
boot_Spear_T2_611_C<-bootnet(Network_Boot_T2_611, nBoots=1000, nCores=8, type="case", verbose=T)
plot(boot_Spear_T2_611_C)
corStability(boot_Spear_T2_611_C)
save(boot_Spear_T2_611_C, file="boot_Spear_T2_611_C")
#for difference
plot(boot_Spear_T2_611, "edge", plot="difference", onlyNonZero = T, order="sample")
plot(boot_Spear_T2_611, "strength")


#continue with all PP at T1 or with 611 PP at T1?
cor(WM_T1_all[lower.tri(WM_T1_all)], WM_T1_611[lower.tri(WM_T1_611)])
#correlation between adjacency matrices of T1_all and T1_611 network is 0.941905; we further investigate the network with 611 PP at T1 and the network with 611 PP at T2

#estimate amount of variance explained by connecting edges in all nodes (611 PP at T1)
type=rep('g', 20)#define variable type as Gaussian
fit1 <- mgmfit(na.omit(Dat1[1:611, 11:30]), type=type, lev=rep(1,20), d=1)#estimate network with mgm
pred1 <- predict.mgm(fit1, na.omit(Dat1[1:611, 11:30]), error.continuous='VarExpl')#save R2 into an object
mean(pred1$error$Error)#mean variance explained across all nodes=0.56435
sd(pred1$error$Error)#0.13
pred1$error$Error#object with R2 of all symptom nodes at T1

#estimate amount of variance explained by connecting edges in all nodes (611 PP at T2)
type2=rep('g', 20)#define variable type as Gaussian
fit2 <- mgmfit(na.omit(Dat2[1:611, 31:50]), type=type2, lev=rep(1,20), d=1)#estimate network with mgm
pred2 <- predict.mgm(fit2, na.omit(Dat2[1:611, 31:50]), error.continuous='VarExpl')#save R2 into an object
mean(pred2$error$Error)#mean variance explained across all nodes=0.5957
sd(pred2$error$Error)#0.11
pred2$error$Error#object with R2 of all symptom nodes at T2

#final networks with 611 PP at T1 and 611 PP at T2, including R2
L_average<-averageLayout(WM_T1_611,WM_T2_611)
SNetwork_T1_611<-qgraph(cor(Dat1[1:611, 11:30], method="spearman"), pie=pred1$error$Error, pieColor = "darkred", pieBorder = 0.25, layout=L_average, graph="glasso", sampleSize=nrow(D_PTSD), cut=0, vsize=4.5, label.cex=1.4, nodeNames=SymptomNames, groups = GroupLabels, palette = "colorblind", labels = LabelsPTSD, legend.cex=.45, maximum=0.46, posCol="darkblue", negCol="red", title="PTSD Network T1")#Spearman_T1_611, strongest edge was 0.32 originally
SNetwork_T2_611<-qgraph(cor(Dat2[1:611, 31:50], method="spearman"), pie=pred2$error$Error, pieColor = "darkred", pieBorder = 0.25, layout=L_average, graph="glasso", sampleSize=nrow(D_PTSD), cut=0, vsize=4.5, label.cex=1.4, nodeNames=SymptomNames, groups = GroupLabels, palette = "colorblind", labels = LabelsPTSD, legend.cex=.45, maximum=0.46, posCol="darkblue", negCol="red", title="PTSD Network T2")#Spearman_T2_611, biggest edge was 0.46 originally

#store networks in pdf
SNetwork_T1_611<-qgraph(cor(Dat1[1:611, 11:30], method="spearman"), pie=pred1$error$Error, pieColor = "darkred", pieBorder = 0.25, layout=L_average, graph="glasso", sampleSize=nrow(D_PTSD), cut=0, vsize=4.5, label.cex=1.4, nodeNames=SymptomNames, groups = GroupLabels, palette = "colorblind", labels = LabelsPTSD, legend.cex=.45, maximum=0.46, posCol="darkblue", negCol="red", title="PTSD Network T1", filetype="pdf", filename="Spearman_T1_611")#strongest edge was 0.32 originally
SNetwork_T2_611<-qgraph(cor(Dat2[1:611, 31:50], method="spearman"), pie=pred2$error$Error, pieColor = "darkred", pieBorder = 0.25, layout=L_average, graph="glasso", sampleSize=nrow(D_PTSD), cut=0, vsize=4.5, label.cex=1.4, nodeNames=SymptomNames, groups = GroupLabels, palette = "colorblind", labels = LabelsPTSD, legend.cex=.45, maximum=0.46, posCol="darkblue", negCol="red", title="PTSD Network T2", filetype="pdf", filename="Spearman_T2_611")#biggest edge was 0.46 originally

#centrality over time
StrengthPlot<-centralityPlot(list(T1=SNetwork_T1_611, T2=SNetwork_T2_611), scale=c("raw0"), labels = LabelsPTSD)
ggsave(filename="Centrality.pdf", plot=StrengthPlot)
#correlation for degree estiamtes at T1 and T2
cor(Degree_T1_611, Degree_T2_611)#0.8610623



#NetworkComparisonTest Preparation
Cov_Spearman_T1_all<- cor(Dat1[ , 11:30], method="spearman")
Cov_Pearson_T1_all<- cor(Dat1[ , 11:30])
cor(Cov_Spearman_T1_all[lower.tri(Cov_Spearman_T1_all)], Cov_Pearson_T1_all[lower.tri(Cov_Pearson_T1_all)])#.9515
#NCT is validated only for Pearson, not Spearman; however, the correlation between Spearman and Pearson covariance matrix is 0.95 in our sample; we perform the NCT based on Pearson 

cor(WM_T1_all[lower.tri(WM_T1_all)], WM_T1_611[lower.tri(WM_T1_611)])
#correlation between adjacency matrices of T1_all and T1_611 network is 0.941905 - we opt for NCT comparing T1_611 with T2_611 (NCT comparing networks of exactly the same people measured twice)

#NetworkComparisonTests
DT1_611<-Dat1[1:611, 11:30]# data set T1, 611 PP
DT2_611<-Dat2[1:611, 31:50]# data set T2, 611 PP

set.seed(1337)
NCT_T1_611_T2_611_Pearson<-NCT(na.omit(DT1_611), na.omit(DT2_611), paired = T, it=5000, binary.data=F, test.edges=T, edges="all", progressbar = T)#gamma default = 0.5 for gaussian
#compare network for T1 and T2 with 611PP
#0.2131173 difference in global strength - p-value = 0.2526
#0.253531 difference in network structure (topology) - p-value = 0.1226
save(NCT_T1_611_T2_611_Pearson, file="NCT_T1_611_T2_611_Pearson.Rdata")

#edges that differ significantly from each other
sum(NCT_T1_611_T2_611_Pearson$einv.pvals$"p-value" < 0.05)#none


